﻿' STEP 1 - Add reference - NEXTWUSB_dotNET_12B.dll
' STEP 2 Imports NEXTWUSB_dotNET_12B
Imports NEXTWUSB_dotNET_12B
Module Module1
    Function checkResponse(slaves As UInteger, column As UInteger, expectValue As UInteger, retry As UInteger) As Boolean
        Dim realValue As UInteger
        Dim retried As UInteger
        realValue = 987654321 'inital data
        For i As Integer = 0 To retry
            NEXTWUSB.ECMUSBRead()
            If (column = 0) Then
                If (slaves = 41) Then
                    realValue = NEXTWUSB.respData(slaves).CMD
                Else
                    realValue = (NEXTWUSB.respData(slaves).CMD & &HFFFF)
                End If
            ElseIf (column = 1) Then
                If (slaves = 0) Then
                    realValue = (NEXTWUSB.respData(slaves).Parm & &HFF)
                Else
                    realValue = (NEXTWUSB.respData(slaves).Parm)
                End If
            ElseIf (column = 2) Then
                realValue = (NEXTWUSB.respData(slaves).Data1)
            ElseIf (column = 3) Then
                realValue = (NEXTWUSB.respData(slaves).Data2)
            End If

            If (realValue = expectValue) Then
                Exit For
            ElseIf (retry = retried) Then
                Console.WriteLine("Fail,realValue= " + realValue.ToString() + " expectValue= " + expectValue.ToString() + " retry = " + retried.ToString())
            Else
                NEXTWUSB.ClearCmdData()
                NEXTWUSB.ECMUSBWrite()
            End If
        Next
        Return (realValue = expectValue)
    End Function

    Sub Main()
        Dim retValue, isOpen As Boolean
        ' STEP 3 - Open the USB Port
        retValue = NEXTWUSB.OpenECMUSB()
        If retValue Then
            isOpen = True
        Else
            Console.WriteLine("Open USB Fail")
            Return
        End If
        System.Threading.Thread.Sleep(100)

        ' STEP 4 - SET_STATE to PreOP
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_STATE
        NEXTWUSB.cmdData(0).Data1 = NEXTWUSB.STATE_PRE_OP ' set state to pre-OP
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(0, 1, 2, 100)

        'STEP 5 - SET_AXIS
        NEXTWUSB.ClearCmdData()
        Dim slaveType() As Char = {"d"c, "i"c, "p"c, "d"c, "d"c, "d"c, "s"c, "i"c, _
                                   "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, _
                                   "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, _
                                   "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, _
                                   "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c, "n"c}
        Dim topology As UInt32
        For i As Integer = 1 To 5
            topology = 0
            For j As Integer = (i - 1) * 8 To (i * 8) - 1
                Select Case slaveType(j)
                    '  Case "d"c  ' "d"c means drive
                    Case "i"c  ' "i"c means io
                        topology += Math.Pow(16, j)
                    Case "p"c  ' "p"c means High Speed Pulse (HSP)
                        topology += Math.Pow(16, j) * 2
					Case "s"c  ' "s"c means Step
                        topology += Math.Pow(16, j) * 3
                End Select
            Next
            NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_AXIS
            NEXTWUSB.cmdData(0).Parm = (i - 1) ' Group 
            NEXTWUSB.cmdData(0).Data1 = topology
            NEXTWUSB.cmdData(0).Data2 = 0
            If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
            System.Threading.Thread.Sleep(2)
        Next

        'STEP 6 - SET_DC
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_DC
        NEXTWUSB.cmdData(0).Parm = 0
        NEXTWUSB.cmdData(0).Data1 = 2000 ' set cycle time to 2000 us
        NEXTWUSB.cmdData(0).Data2 = &HFFFF ' Auto offet adjustment
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(2)

        ' STEP 7 - Set DRIVE_MODE
        NEXTWUSB.ClearCmdData()
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "d"c OrElse slaveType(i - 1) = "p"c OrElse slaveType(i - 1) = "s"c) Then
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.DRIVE_MODE
                NEXTWUSB.cmdData(i).Data1 = NEXTWUSB.CSP_MODE ' set to CSP mode
                NEXTWUSB.cmdData(i).Data2 = NEXTWUSB.DCSYNC ' set using DC sync
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(2)

        ' STEP 7.1 - Read Slave 1 Object 0x1018:01 (Vender ID) via SDO (This step could be skipped)
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(41).CMD = NEXTWUSB.SDO_RD
        NEXTWUSB.cmdData(41).Parm = 1 ' slave index (1~40)
        NEXTWUSB.cmdData(41).Data1 = &H10180001 ' Object Index, subIndex
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(41, 0, NEXTWUSB.SDO_RD, 1000)  ' check Response
        checkResponse(41, 1, 1, 1000)
        checkResponse(41, 2, &H10180001, 1000)
        NEXTWUSB.ECMUSBRead()
        Console.WriteLine("Read Slave 1 Vender ID via SDO: " & NEXTWUSB.respData(41).Data2)

        ' STEP 7.2 - Write Slave 1 Object 0x1F01:02 (Sync error count limit) via SDO (This step could be skipped)
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(41).CMD = NEXTWUSB.SDO_WR
        NEXTWUSB.cmdData(41).Parm = &H401 ' Size(1~4 Byte), slave index (1~40)
        NEXTWUSB.cmdData(41).Data1 = &H1F010002 ' Object Index, subIndex
        NEXTWUSB.cmdData(41).Data2 = 9 ' Value
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(41, 0, NEXTWUSB.SDO_WR, 1000)  ' check Response

        ' STEP 8 - SET_STATE to SafeOP (This step could be skipped)
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_STATE
        NEXTWUSB.cmdData(0).Data1 = NEXTWUSB.STATE_SAFE_OP  ' // set state to Safe-OP
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(0, 1, 4, 1000)  ' wait for enter to safe op mode

        ' STEP 9 - SET_STATE to OP
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_STATE
        NEXTWUSB.cmdData(0).Data1 = NEXTWUSB.STATE_OPERATIONAL ' set state to OP
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(0, 1, 8, 1000) 'wait for enter to op mode

        ' STEP 10 - SET Servo On
        NEXTWUSB.ClearCmdData()
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "d"c OrElse slaveType(i - 1) = "p"c OrElse slaveType(i - 1) = "s"c) Then 'set drive or HSP slave servo on
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.SV_ON
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(400)  ' wait for servo on 

        ' STEP 11- Read Current Position
        NEXTWUSB.ECMUSBRead()
        Dim TargerPosition(NEXTWUSB.DEF_MA_MAX) As Integer
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 1
            Dim temp = BitConverter.GetBytes((NEXTWUSB.respData(i).Data1))
            TargerPosition(i) = BitConverter.ToInt32(temp, 0) ' Change UInt to Int32
        Next

        ' STEP 12- SET Cycle Synchronized Position
        System.Threading.Thread.Sleep(1000) ' wait until motion stop totally
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.ECMUSBRead()
        Dim showSDOValue As Boolean = False
        Console.WriteLine("Current Position: Slave 1 = " + NEXTWUSB.respData(1).Data1.ToString() +
            " Slave 2 = " + NEXTWUSB.respData(2).Data1.ToString() + ", Slave 3 = " + NEXTWUSB.respData(3).Data1.ToString())
        For i As Integer = 1 To 1000     'send 1000 times, let motor move 1000 * 100 pulse
            For j As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
                If (slaveType(j - 1) = "d"c OrElse slaveType(j - 1) = "p"c OrElse slaveType(j - 1) = "s"c) Then 'Set drive or HSP slave Cycle Synchronized Position
                    TargerPosition(j) += 3 ' set to move 100 pulse in one cycle time
                    NEXTWUSB.cmdData(j).CMD = NEXTWUSB.CSP
                    Dim temp = BitConverter.GetBytes(TargerPosition(j))
                    NEXTWUSB.cmdData(j).Data1 = BitConverter.ToUInt32(temp, 0) ' Change Int32 to UInt
                End If
            Next
            If (i = 100) Then
                'Read slave 1 Object 0x60F4:0 (Following Error Actual Value) via SDO
                NEXTWUSB.cmdData(41).CMD = NEXTWUSB.SDO_RD
                NEXTWUSB.cmdData(41).Parm = 1 ' slave index (1~40)
                NEXTWUSB.cmdData(41).Data1 = &H60F40000  ' Object Index, subIndex
                showSDOValue = True
            End If

            If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
            If (i Mod 2 = 0) Then ' USBWrite 2 times and USBRead 1 time
                NEXTWUSB.ECMUSBRead()
                If ((NEXTWUSB.respData(0).Data2 Mod 65536) < 80) Then ' If FIFO remaining < 80 then sleep 10ms
                    If (showSDOValue) Then
                        If (NEXTWUSB.respData(41).CMD = NEXTWUSB.SDO_RD AndAlso NEXTWUSB.respData(41).Parm = 1 AndAlso NEXTWUSB.respData(41).Data1 = &H60F40000) Then
                            Console.WriteLine("Read Slave 1 Object 0x60F4:0 (Following Error Value) via SDO: " & NEXTWUSB.respData(41).Data2)
                            showSDOValue = False
                        End If
                    End If
                    System.Threading.Thread.Sleep(10)
                End If
            End If
        Next
        System.Threading.Thread.Sleep(3000)  ' wait until motion stop totally
        NEXTWUSB.ECMUSBRead()
        Console.WriteLine("Current Position: Slave 1 = " + NEXTWUSB.respData(1).Data1.ToString() +
            " Slave 2 = " + NEXTWUSB.respData(2).Data1.ToString() + ", Slave 3 = " + NEXTWUSB.respData(3).Data1.ToString())

        ' STEP 13- SET Servo Off
        NEXTWUSB.ClearCmdData()
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "d"c OrElse slaveType(i - 1) = "p"c OrElse slaveType(i - 1) = "s"c) Then 'Set drive or HSP slave servo off
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.SV_OFF
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(100)

        ' STEP 14- Read Input
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.LIO_RD 'ECM can read Local Input
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "i"c) Then 'IO slave can read input
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.IO_RD
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(10)
        NEXTWUSB.ECMUSBRead()
        Console.WriteLine("ECM Local Input Value =" + NEXTWUSB.respData(0).Data1.ToString())
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "i"c) Then 'IO slave can read input
                Console.WriteLine("Current Position: Slave " + i.ToString() + " = " + NEXTWUSB.respData(i).Data1.ToString())
            End If
            If (slaveType(i - 1) = "p"c) Then 'HSP slave can read input
                Console.WriteLine("Current Position: Slave " + i.ToString() + " = " + (NEXTWUSB.respData(i).Data2 And &HFF).ToString())
            End If
        Next
        System.Threading.Thread.Sleep(100)

        ' STEP 15- Write Output
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.LIO_WR 'ECM can set Local Output
        NEXTWUSB.cmdData(0).Data1 = &HF  'Output Value
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "i"c) Then ' IO slave can set Output
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.IO_WR
                NEXTWUSB.cmdData(i).Data1 = &HFFFF
            End If
            If (slaveType(i - 1) = "p"c) Then ' HSP slave can set Output
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.CSP
                NEXTWUSB.cmdData(i).Data2 = NEXTWUSB.cmdData(i).Data2 Or &HFF
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(5000)
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.LIO_WR 'ECM can set Local Output
        NEXTWUSB.cmdData(0).Data1 = &H0  'Output Value
        For i As Integer = 1 To NEXTWUSB.DEF_MA_MAX - 2
            If (slaveType(i - 1) = "i"c) Then 'only drive slave can set servo off
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.IO_WR
                NEXTWUSB.cmdData(i).Data1 = &H0
            End If
            If (slaveType(i - 1) = "p"c) Then ' HSP slave can set Output
                NEXTWUSB.cmdData(i).CMD = NEXTWUSB.CSP
                NEXTWUSB.cmdData(i).Data2 = NEXTWUSB.cmdData(i).Data2 And &HFF00
            End If
        Next
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        System.Threading.Thread.Sleep(1000)

        ' SETP 16- SET_STATE to Init
        NEXTWUSB.ClearCmdData()
        NEXTWUSB.cmdData(0).CMD = NEXTWUSB.SET_STATE
        NEXTWUSB.cmdData(0).Data1 = NEXTWUSB.STATE_INIT
        If (Not isOpen OrElse Not NEXTWUSB.ECMUSBWrite()) Then Console.WriteLine("Write Error")
        checkResponse(0, 1, 1, 1000)
        ' STEP 17- Close USB
        NEXTWUSB.CloseECMUSB()

    End Sub

End Module
